==> lsedit-7.0.36.zip: lsedit/AttributeValueItem.java <==
package lsedit;

import java.io.IOException;
import java.io.PrintStream;

public class AttributeValueItem extends Object 
{
	// Attribute values can be single nodes, a linked list of nodes, or they can be nested lists.
	// 

	public String				value;
	public AttributeRecord		nestedAttributeRecord;
	public AttributeValueItem	next;
	public AttributeValueItem	nextList;

	// If any of these characters appear in an id or an attribute 
	// value, this string should be output quoted. 
	//

	protected static final String specialChars = "\n\"\\ /,:(){}^<>"; 

	public static String qt(String str) 
	{
		if (str.length() == 0) {
			return "\"\"";
		}
		for (int i = 0; i < specialChars.length(); i++) {
			if (str.indexOf(specialChars.charAt(i)) >= 0) {
				if (i < 2) {
					String	ret = "\"";
					char	c;

					c = 0;
					for (i = 0; i < str.length(); ++i) {
						c    = str.charAt(i);
						switch (c) {
						case '\\':
							ret += c;
							if (++i < str.length()) {
								ret += str.charAt(i);
							}
							continue;
						case '\n':
							c    = 'n';
						case '"':
							ret += '\\';
							break;
						}
						ret += c;
					}
					return ret + '"';
				}
				return "\"" + str + "\"";
			}

		}
		return str;
	}


	// ----------------- 
	// Protected methods
	// -----------------

	protected void printList(PrintStream ps) throws IOException
	{
		AttributeValueItem avi = this;	 

		while(avi != null) {
			if (avi.next != null) {
				ps.print(qt(avi.value) + " ");
			} else {
				ps.print(qt(avi.value)); 
			}
			avi = avi.next; 
		}
	}



	// -------------- 
	// Public methods
	// -------------- 

	public AttributeValueItem(String value) 
	{
		this.value = value;
	}

	public AttributeValueItem(AttributeRecord ar) 
	{
		nestedAttributeRecord = ar;
	}

	public Object clone() 
	{
		AttributeValueItem av;

		if (nestedAttributeRecord != null) {
			av = new AttributeValueItem((AttributeRecord) nestedAttributeRecord.clone()); 
		} else {
			av = new AttributeValueItem(value);
		}

		if (next != null) {
			av.next = (AttributeValueItem) next.clone();
		}

		if (nextList != null) {
			av.nextList = (AttributeValueItem) nextList.clone();
		}
		return av; 
	}

	public String toString() {

		AttributeValueItem avi;
		String			   ret;	  

		if (next != null) {
			ret = "(";
		} else {
			ret = "";
		}
		for (avi = this; avi != null; avi = avi.next) {
			if (avi != this) {
				ret += " ";
			}
			ret += qt(avi.value);
		}
		if (next != null) {
			ret += ")";
		}
		return(ret);
	}

	public void writeAttribute(PrintStream ps) throws IOException {

		if (next != null) {
			ps.print("(");
			printList(ps);
			ps.print(")\n"); 
		} else if (nestedAttributeRecord != null) {
			// Nested case must be handled!
			ps.print("{\n"); 
			ps.print("}\n"); 
		} else {
			// Single string value is output
			ps.print(qt(value) + "\n"); 
		}
	}

	public boolean equals(Object other)
	{
		if (other != null && other instanceof AttributeValueItem) {
			if (nestedAttributeRecord == null && ((AttributeValueItem) other).nestedAttributeRecord == null) {
				if (value != null && value.equals(((AttributeValueItem) other).value)) {
					if (next != null) {
						return(next.equals(((AttributeValueItem) other).next));
					} else if (((AttributeValueItem) other).next == null) {
						return(true);
		}	}	}	}
		return(false);
	}

	public RealPoint parsePoint() {

		if (value == null || next == null || next.value == null) {
			return new RealPoint(0.0, 0.0);
		}
		double v1 = (new Double(value)).doubleValue();
		double v2 = (new Double(next.value)).doubleValue();
		return new RealPoint(v1, v2);
	}
}

==> lsedit-7.0.42.zip: lsedit/AttributeValueItem.java <==
package lsedit;

import java.io.IOException;
import java.io.PrintStream;

public class AttributeValueItem extends Object 
{
	// Attribute values can be single nodes, or a linked list of nodes.
	// 

	public String				value;
	public AttributeValueItem	next;		// Next item within this attribute value (x y z)
	public AttributeValueItem	nextList;	// Next attribute value in the sublist of terms ( ... (x y z) ...)

	// If any of these characters appear in an id or an attribute 
	// value, this string should be output quoted. 
	//

	protected static final String specialChars = "\n\"\\ /,:(){}^<>"; 

	public static String qt(String str) 
	{
		if (str.length() == 0) {
			return "\"\"";
		}
		for (int i = 0; i < specialChars.length(); i++) {
			if (str.indexOf(specialChars.charAt(i)) >= 0) {
				if (i < 2) {
					String	ret = "\"";
					char	c;

					c = 0;
					for (i = 0; i < str.length(); ++i) {
						c    = str.charAt(i);
						switch (c) {
						case '\\':
							ret += c;
							if (++i < str.length()) {
								ret += str.charAt(i);
							}
							continue;
						case '\n':
							c    = 'n';
						case '"':
							ret += '\\';
							break;
						}
						ret += c;
					}
					return ret + '"';
				}
				return "\"" + str + "\"";
			}

		}
		return str;
	}

	// -------------- 
	// Public methods
	// -------------- 

	public AttributeValueItem(String value) 
	{
		this.value = value;
	}

	public Object clone() 
	{
		AttributeValueItem av = new AttributeValueItem(value);

		if (next != null) {
			av.next = (AttributeValueItem) next.clone();
		}

		if (nextList != null) {
			av.nextList = (AttributeValueItem) nextList.clone();
		}
		return av; 
	}

	public String toString() {

		AttributeValueItem avi;
		String			   ret;	  

		if (next != null) {
			ret = "(";
		} else {
			ret = "";
		}
		for (avi = this; avi != null; avi = avi.next) {
			if (avi != this) {
				ret += " ";
			}
			ret += qt(avi.value);
		}
		if (next != null) {
			ret += ")";
		}
		return(ret);
	}

	public void writeAttribute(PrintStream ps) throws IOException {

		if (next != null) {
			AttributeValueItem avi;

			ps.print("(");

			for (avi = this; avi != null; avi = avi.next) {
				if (avi != this) {
					ps.print(" ");
				}
				if (avi.nextList == null) {
					ps.print(qt(avi.value)); 
				} else {
					ps.print("(");
					for (AttributeValueItem avi1 = avi; avi1 != null; avi1 = avi1.nextList) {
						if (avi1 != avi) {
							ps.print(" ");
						}
						ps.print(qt(avi1.value));
					}
					ps.print(")");
			}	}
			ps.print(")\n"); 
		} else {
			// Single string value is output
			ps.print(qt(value) + "\n"); 
		}
	}

	public boolean equals(Object other)
	{
		if (other != null && other instanceof AttributeValueItem) {
			if (value != null && value.equals(((AttributeValueItem) other).value)) {
				if (next != null) {
					return(next.equals(((AttributeValueItem) other).next));
				} else if (((AttributeValueItem) other).next == null) {
					return(true);
		}	}	}
		return(false);
	}

	public RealPoint parsePoint() {

		if (value == null || next == null || next.value == null) {
			return new RealPoint(0.0, 0.0);
		}
		double v1 = (new Double(value)).doubleValue();
		double v2 = (new Double(next.value)).doubleValue();
		return new RealPoint(v1, v2);
	}
}
